# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'SitemapEntry.priority'
        db.alter_column(u'sitemapper_sitemapentry', 'priority', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=2, decimal_places=1))

        # Changing field 'SitemapEntry.changefreq'
        db.alter_column(u'sitemapper_sitemapentry', 'changefreq', self.gf('django.db.models.fields.PositiveSmallIntegerField')(null=True))

    def backwards(self, orm):

        # Changing field 'SitemapEntry.priority'
        db.alter_column(u'sitemapper_sitemapentry', 'priority', self.gf('django.db.models.fields.DecimalField')(default='0.5', max_digits=2, decimal_places=1))

        # Changing field 'SitemapEntry.changefreq'
        db.alter_column(u'sitemapper_sitemapentry', 'changefreq', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=30))

    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'sitemapper.sitemapentry': {
            'Meta': {'unique_together': "(['content_type', 'object_id'],)", 'object_name': 'SitemapEntry'},
            'changefreq': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'priority': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '2', 'decimal_places': '1', 'blank': 'True'})
        }
    }

    complete_apps = ['sitemapper']


from django.contrib import admin
from .models import Demo

# Step 1: You'll need to import the admin-inline
from sitemapper.admin import SitemapEntryInline


class DemoAdmin(admin.ModelAdmin):
    list_display = ('title', 'slug', 'modified')
    readonly_fields = ('modified',)
    prepopulated_fields = {'slug': ('title',)}

    # Step 2: Add the admin-inline to the inlines of your admin class
    inlines = [SitemapEntryInline]

admin.site.register(Demo, DemoAdmin)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Demo'
        db.create_table(u'demoapp_demo', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=60)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=50)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal(u'demoapp', ['Demo'])


    def backwards(self, orm):
        # Deleting model 'Demo'
        db.delete_table(u'demoapp_demo')


    models = {
        u'demoapp.demo': {
            'Meta': {'object_name': 'Demo'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '60'})
        }
    }

    complete_apps = ['demoapp']


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from decimal import Decimal


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SitemapEntry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.PositiveIntegerField()),
                ('changefreq', models.PositiveSmallIntegerField(verbose_name='Change frequency', choices=[(10, b'always'), (20, b'hourly'), (30, b'daily'), (40, b'weekly'), (50, b'monthly'), (60, b'yearly'), (70, b'never')])),
                ('priority', models.DecimalField(verbose_name='priority', max_digits=2, decimal_places=1, choices=[(Decimal('0.0'), b'0.0'), (Decimal('0.1'), b'0.1'), (Decimal('0.2'), b'0.2'), (Decimal('0.3'), b'0.3'), (Decimal('0.4'), b'0.4'), (Decimal('0.5'), b'0.5'), (Decimal('0.6'), b'0.6'), (Decimal('0.7'), b'0.7'), (Decimal('0.8'), b'0.8'), (Decimal('0.9'), b'0.9'), (Decimal('1.0'), b'1.0')])),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Sitemap entry',
                'verbose_name_plural': 'Sitemap entries',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='sitemapentry',
            unique_together=set([('content_type', 'object_id')]),
        ),
    ]



"""
Test support harness to make setup.py test work.
"""

import sys
from django.conf import settings

settings.configure(
    DATABASES={
        'default': {'ENGINE': 'django.db.backends.sqlite3', 'NAME': ':memory;'}
        },
    MIDDLEWARE_CLASSES=(
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        ),
    INSTALLED_APPS=[
        'django.contrib.contenttypes',
        'django.contrib.sitemaps',
        'sitemapper',
        'sitemapper.tests'
        ],
    ROOT_URLCONF='sitemapper.tests.urls',
    USE_TZ = True
    )


def runtests():
    import django
    import django.test.utils

    print('# runtests: Django version: {}'.format(django.VERSION))

    if django.VERSION[:2] < (1, 6):
        print('# runtests: Using DiscoverRunner')
        settings.INSTALLED_APPS.append('discover_runner')
        settings.TEST_RUNNER = 'discover_runner.DiscoverRunner'

    if hasattr(django, 'setup'):
        django.setup()

    runner_class = django.test.utils.get_runner(settings)
    test_runner = runner_class(verbosity=1, interactive=True)
    failures = test_runner.run_tests(['sitemapper'])
    sys.exit(failures)



# Django
from django.contrib import sitemaps

# Local
from .models import SitemapEntry


class SitemapBase(sitemaps.Sitemap):
    """The root class for sitemapper.Sitemaps.

    Inherits from django.contrib.sitemaps.Sitemap and overrides the
    Sitemap.items() method.

    In this class calling items() does two things:

    1. It creates a private attribute ``_entries`` containing a dictionary of
       sitemapper.SitemapEntry objects having the same ContentType as the
       supplied queryset's model. This dictionary is keyed to the object_id of
       each SitemapEntry instance.

    2. It then returns the queryset.

    Attributes:

      queryset (queryset): A queryset containing the objects to appear in
        the sitemap.

      _entries (dict): A private variable populated by the items() method and
        containing SitemapEntries matching the ContentType and object_id of the
        queryset above.
    """

    def _get_entries_for_model(self, model):
        """
        Return a dictionary of sitemapper.SitemapEntry objects whose
        ContentType matches ``model``. The dictionary is keyed to the
        SitemapEntry's object_id.

        Args:

          model (class): This is the model class or instance to be filtered on.
        """
        entries = SitemapEntry.objects.get_for_model(model)
        return {entry.object_id: entry for entry in entries}

    def items(self):
        """
        Assign the result of _get_entries_for_model(model) to the private
        attribute _entries, and return the queryset attribute.
        """
        self._entries = self._get_entries_for_model(self.queryset.model)
        return self.queryset.filter()


class Sitemap(SitemapBase):

    def changefreq(self, item):
        """
        Return one of three values:

        1. the SitemapEntry.changefreq related to ``item``, if set;
        2. or, the default_changefreq, if set;
        3. or, None

        Args:
          item (model instance): An member instance of self.queryset.
        """
        default = getattr(self, 'default_changefreq', None)
        try:
            changefreq = self._entries.get(item.id).get_changefreq_display()
        except AttributeError:
            changefreq = None
        return changefreq or default

    def priority(self, item):
        """
        Return one of three values:

        1. the SitemapEntry.priority related to ``item``, if set;
        2. or, the default_priority, if set;
        3. or, None

        Args:
          item (model instance): An member instance of self.queryset.
        """
        default = getattr(self, 'default_priority', None)
        try:
            priority = self._entries.get(item.id).get_priority_display()
        except AttributeError:
            priority = None
        return priority or default


class GenericSitemap(Sitemap):
    """
    A sub-class of Sitemapper which emulates
    django.contrib.sitemaps.GenericSitemap, except that any supplied
    ``priority`` or ``changefreq`` args set the corresponding default_*
    attributes.

    Args:

      info_dict (dictionary): Optional dictionary which may contain
        ``date_field`` and, or, ``queryset``.

      priority (string or float): Optional value assigned internally to
        ``default_priority`` and used **only** if not overridden by a
        SitemapEntry.

      changefreq (string): Optional value assigned internally to
        ``default_changefreq`` and used **only** if not overridden by a
        SitemapEntry. Must be one of: 'always' 'hourly', 'daily', 'weekly',
        'monthly', 'yearly' or 'never'.
    """

    def __init__(self, info_dict={}, priority=None, changefreq=None):

        queryset = info_dict.get('queryset', None)
        if queryset:
            self.queryset = queryset

        date_field = info_dict.get('date_field', None)
        if date_field:
            self.date_field = date_field

        if priority:
            self.default_priority = priority

        if changefreq:
            self.default_changefreq = changefreq

    def lastmod(self, item):
        if self.date_field is not None:
            return getattr(item, self.date_field)
        return None



import warnings
warnings.warn(
    "'sitemapper.mixins.Sitemap' is deprecated, and will be removed in"
    " future versions; use 'sitemapper.sitemaps.Sitemap' instead.",
    DeprecationWarning
    )

from sitemapper.sitemaps import Sitemap  # noqa



from django.conf.urls import url
from django.contrib.sitemaps.views import sitemap
from django.http import HttpResponse
from django.views.generic import base

from sitemapper.tests.test_sitemap import SitemapBasic, SitemapWithDefaults


class TestView(base.View):
    def get(self, request, *args, **kwargs):
        return HttpResponse('Hello Test World!')


basic_dict = {
    'tests': SitemapBasic,
    }

withdefaults_dict = {
    'tests': SitemapWithDefaults,
    }


urlpatterns = [

    url(r'^(?P<slug>[-_\w]+)/$',
        TestView.as_view(),
        name='sitemapper.tests.views.detail'
        ),

    url(r'^sitemap-basic\.xml',
        sitemap,
        {'sitemaps': basic_dict},
        name='sitemap.basic'
        ),

    url(r'^sitemap-withdefaults\.xml',
        sitemap,
        {'sitemaps': withdefaults_dict},
        name='sitemap.with.defaults'
        ),

    ]



from django.conf.urls import url
from django.views.generic.detail import DetailView
from .models import Demo


urlpatterns = [

    url(r'^(?P<slug>[-_\w]+)/$',
        DetailView.as_view(queryset=Demo.objects.all()),
        name='demoapp.views.detail'
        ),

    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from decimal import Decimal


class Migration(migrations.Migration):

    dependencies = [
        ('sitemapper', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sitemapentry',
            name='changefreq',
            field=models.PositiveSmallIntegerField(blank=True, null=True, verbose_name='Change frequency', choices=[(10, b'always'), (20, b'hourly'), (30, b'daily'), (40, b'weekly'), (50, b'monthly'), (60, b'yearly'), (70, b'never')]),
        ),
        migrations.AlterField(
            model_name='sitemapentry',
            name='priority',
            field=models.DecimalField(decimal_places=1, choices=[(Decimal('0'), b'0.0'), (Decimal('0.1'), b'0.1'), (Decimal('0.2'), b'0.2'), (Decimal('0.3'), b'0.3'), (Decimal('0.4'), b'0.4'), (Decimal('0.5'), b'0.5'), (Decimal('0.6'), b'0.6'), (Decimal('0.7'), b'0.7'), (Decimal('0.8'), b'0.8'), (Decimal('0.9'), b'0.9'), (Decimal('1'), b'1.0')], max_digits=2, blank=True, null=True, verbose_name='priority'),
        ),
    ]



# Django
from django.contrib import admin
try:
    from django.contrib.contenttypes.admin import GenericStackedInline
except ImportError:
    from django.contrib.contenttypes.generic import GenericStackedInline

# Local
from .models import SitemapEntry


class SitemapEntryInline(GenericStackedInline):
    model = SitemapEntry
    max_num = 1


class SitemapEntryAdmin(admin.ModelAdmin):
    list_display = (
        'content_object',
        'content_type',
        'object_id',
        'changefreq',
        'priority',
        )

admin.site.register(SitemapEntry, SitemapEntryAdmin)



from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext as _


@python_2_unicode_compatible
class Demo(models.Model):

    title = models.CharField(_('title'), max_length=60)
    slug = models.SlugField(_('slug'), unique=True)
    modified = models.DateTimeField(_('modified'), auto_now=True)

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('demoapp.views.detail', kwargs={
            'slug': self.slug,
            })



import datetime
from django.test import TestCase, Client
from django.utils import timezone
from django.core.urlresolvers import reverse
from sitemapper.tests.models import Demo
from sitemapper.sitemaps import Sitemap


class SitemapBasic(Sitemap):
    queryset = Demo.objects.all()

    def lastmod(self, obj):
        return obj.modified


class SitemapWithDefaults(SitemapBasic):
    default_changefreq = 'weekly'
    default_priority = 0.5


expected_dicts = {

    'http://testserver/default-sitemap/': {
        'priority': '0.5',  # This is a default_priority
        'lastmod': datetime.datetime(
            2014, 10, 5, 16, 32, 19, 228000, tzinfo=timezone.utc
            ),
        'changefreq': 'weekly',  # This is a default_changefreq
        },

    'http://testserver/frequent-sitemap/': {
        'priority': '0.9',
        'lastmod': datetime.datetime(
            2014, 10, 5, 16, 31, 36, 60000, tzinfo=timezone.utc
            ),
        'changefreq': u'hourly',
        },

    'http://testserver/infrequent-sitemap/': {
        'priority': '0.1',
        'lastmod': datetime.datetime(
            2014, 10, 5, 16, 32, 3, 763000, tzinfo=timezone.utc
            ),
        'changefreq': u'yearly',
        }

    }


class SitemapTestCase(TestCase):

    fixtures = ['test.json']

    def setUp(self):
        self.client = Client()

    def test_response_basic(self):
        response = self.client.get(reverse(
            'sitemap.basic'
            ))
        self.assertEqual(response.status_code, 200)
        context = response.context['urlset']
        for got in context:
            expected = expected_dicts[got['location']]

            if got['item'].slug == 'default-sitemap':
                self.assertFalse(got['changefreq'])
                self.assertFalse(got['priority'])
                self.assertEqual(got['lastmod'], expected['lastmod'])
            else:
                for field in ('changefreq', 'lastmod', 'priority'):
                    self.assertEqual(got[field], expected[field])

    def test_response_withdefaults(self):
        response = self.client.get(reverse(
            'sitemap.with.defaults'
            ))
        self.assertEqual(response.status_code, 200)
        context = response.context['urlset']
        for got in context:
            expected = expected_dicts[got['location']]

            for field in ('changefreq', 'lastmod', 'priority'):
                self.assertEqual(got[field], expected[field])



from django.conf.urls import include, url
from django.contrib import admin

from django.contrib.sitemaps.views import sitemap
from demoapp.sitemaps import DemoSitemap

admin.autodiscover()

sitemaps = {
    'demos': DemoSitemap,
    }

urlpatterns = [
    # Examples:
    # url(r'^$', 'example.views.home', name='home'),
    # url(r'^blog/', include('blog.urls')),

    url(r'^admin/', include(admin.site.urls)),
    url(r'^demoapp/', include('demoapp.urls')),

    url(r'^sitemap\.xml', sitemap, {'sitemaps': sitemaps},
        name='django.contrib.sitemaps.views.sitemap'),

    ]



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'SitemapEntry'
        db.create_table(u'sitemapper_sitemapentry', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('changefreq', self.gf('django.db.models.fields.PositiveSmallIntegerField')()),
            ('priority', self.gf('django.db.models.fields.DecimalField')(max_digits=2, decimal_places=1)),
        ))
        db.send_create_signal(u'sitemapper', ['SitemapEntry'])

        # Adding unique constraint on 'SitemapEntry', fields ['content_type', 'object_id']
        db.create_unique(u'sitemapper_sitemapentry', ['content_type_id', 'object_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'SitemapEntry', fields ['content_type', 'object_id']
        db.delete_unique(u'sitemapper_sitemapentry', ['content_type_id', 'object_id'])

        # Deleting model 'SitemapEntry'
        db.delete_table(u'sitemapper_sitemapentry')


    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'sitemapper.sitemapentry': {
            'Meta': {'unique_together': "(['content_type', 'object_id'],)", 'object_name': 'SitemapEntry'},
            'changefreq': ('django.db.models.fields.PositiveSmallIntegerField', [], {}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'priority': ('django.db.models.fields.DecimalField', [], {'max_digits': '2', 'decimal_places': '1'})
        }
    }

    complete_apps = ['sitemapper']


# Python
import os
from setuptools import setup

# Local
from sitemapper import __version__, __project__, __doc__

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name=__project__,
    version=__version__,
    author='Mike Hurt',
    author_email='mike@mhtechnical.net',
    description=__doc__,
    long_description=README,
    license='MIT License',
    url='https://bitbucket.org/mhurt/django-sitemapper/',
    install_requires=['Django >= 1.4.2'],
    keywords='python django sitemap.xml',
    packages=['sitemapper', 'sitemapper.migrations', 'sitemapper.tests'],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
    test_suite='sitemapper.tests.run.runtests',
    tests_require=[
        'django-discover-runner',
        'Django >= 1.4.2',
        ],
    )



from sitemapper.sitemaps import Sitemap
from .models import Demo


class DemoSitemap(Sitemap):
    default_changefreq = 'weekly'
    default_priority = 0.5
    queryset = Demo.objects.all()

    def lastmod(self, obj):
        return obj.modified



# Python
from decimal import Decimal

# Django
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext as _

try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey


class SitemapEntryManager(models.Manager):

    def get_for_model(self, model):
        """
        Return all entries for a given model.
        """
        ctype = ContentType.objects.get_for_model(model)
        return self.filter(content_type=ctype)


@python_2_unicode_compatible
class SitemapEntry(models.Model):

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    CHANGEFREQS = (
        (10, 'always'),
        (20, 'hourly'),
        (30, 'daily'),
        (40, 'weekly'),
        (50, 'monthly'),
        (60, 'yearly'),
        (70, 'never'),
    )
    changefreq = models.PositiveSmallIntegerField(
        _('Change frequency'),
        choices=CHANGEFREQS,
        blank=True, null=True,
        )

    PRIORITY_CHOICES = (
        (Decimal('0'), '0.0'),
        (Decimal('0.1'), '0.1'),
        (Decimal('0.2'), '0.2'),
        (Decimal('0.3'), '0.3'),
        (Decimal('0.4'), '0.4'),
        (Decimal('0.5'), '0.5'),
        (Decimal('0.6'), '0.6'),
        (Decimal('0.7'), '0.7'),
        (Decimal('0.8'), '0.8'),
        (Decimal('0.9'), '0.9'),
        (Decimal('1'), '1.0'),
        )
    priority = models.DecimalField(
        _('priority'),
        max_digits=2, decimal_places=1,
        choices=PRIORITY_CHOICES,
        blank=True, null=True,
        )

    # Managers
    objects = SitemapEntryManager()

    class Meta:
        unique_together = (['content_type', 'object_id'],)
        verbose_name = _('Sitemap entry')
        verbose_name_plural = _('Sitemap entries')

    def __str__(self):
        return str(self.content_object)
